// Int32ArrayMessage.cc
/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPF is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include "DLList.h"
#include "DataFreeList.h"
#include "Int32ArrayMessage.h"

DLList* Int32ArrayMessage::pFreeList = 0;

int32 Int32ArrayMessage:: toBuffer(char **buffer) const
{
    int32 nbytes;

    nbytes = StandardFieldsMessage::toBuffer(buffer);
    nbytes += MPF::toBuffer(buffer,(int32)(size/sizeof(int32)));
    nbytes += MPF::toBuffer(buffer,(const void *)value,size);
    return(nbytes);
}

int32 Int32ArrayMessage:: fromBuffer(const char **buffer)
{
    int32 nbytes=0;
    int32 length=0;
    
    nbytes = StandardFieldsMessage::fromBuffer(buffer);
    nbytes += MPF::fromBuffer(buffer,&length);
    size = length*sizeof(int32);
    if(maxSize==0) allocValue(size);
    if(maxSize<size) {
	printf("Int32ArrayMessage:: fromBuffer maxSize<size VERY bad\n");
	exit(-1);
    }
    nbytes += MPF::fromBuffer(buffer,(void *)value,size);
    return(nbytes);
}

int32 Int32ArrayMessage:: fromBufferSwitch(const char **buffer)
{
    int32 nbytes=0;
    int32 length=0;

    nbytes = StandardFieldsMessage::fromBufferSwitch(buffer);
    nbytes += MPF::fromBufferSwitch(buffer,&length);
    size = length*sizeof(int32);
    if(maxSize==0) allocValue(size);
    if(maxSize<size) {
	printf("Int32ArrayMessage:: fromBuffer maxSize<size VERY bad\n");
	exit(-1);
    }
    for(int i=0; i<length; i++) 
      nbytes += MPF::fromBufferSwitch(buffer,&value[i]);

    return(nbytes);
}

void Int32ArrayMessage:: print() const
{
    int32 length = size/sizeof(int32);

    StandardFieldsMessage::print();
    printf("maxLength %d length %d value %p",
	   (int32)(maxSize/sizeof(int32)),length,value);
    for(int i=0; i<length; i++) printf(" %d",value[i]);
    printf("\n");
}

void *Int32ArrayMessage:: operator new(size_t size)
{
    if(size==sizeof(Int32ArrayMessage)) return MPF:: allocate(&pFreeList,size);
    printf("Int32ArrayMessage new size !=sizeof(Int32ArrayMessage)\n");
    return(0);
}
 
void Int32ArrayMessage:: operator delete(void *p, size_t size)
{
    if(size==sizeof(Int32ArrayMessage)) {
	MPF::free(pFreeList,p,size);
	return;
    }
    printf("Int32ArrayMessage delete size !=sizeof(Int32ArrayMessage)\n");
}

void Int32ArrayMessage::setLength(int32 length)
{
    if((int32)(length*sizeof(int32))>maxSize) {
	printf("Int32ArrayMessage::setLength length %d > maxLength %d\n",
	    length,(int32)(maxSize*sizeof(int32)));
	this->size = maxSize;
    } else 
	this->size = length*sizeof(int32);
}

void Int32ArrayMessage::allocValue(int32 maxLength)
{
    DataFreeList::alloc(maxLength*sizeof(int32),(void **)&value,&this->maxSize);
}

void Int32ArrayMessage::freeValue()
{
    if(maxSize==0) return;
    DataFreeList::free((void *)value,maxSize);
    maxSize=0; value=0;
}
